#######################
#This file runs the analysis of the asthma data in the QBC paper
#######################

rm(list=ls())
#read in some ~helpful functions
source('Functions/functions.r',local=TRUE)
#read in the data
data=as.matrix(read.table('Data/Asthma_data.txt'))
M=data[1,1] #number of spatial regions
Nt=data[1,2] #number of timepoints
data=data[-1,] #remove first row (it's junk after M and Nt)
p=dim(data)[2]-1 #get dimension of beta-vector
x=as.matrix(data[,1:p]) #fill in "x" matrix (covariates)
x[,2:5]=apply(x[,2:5],2,standard) #standarize continuous covariates
Y=as.matrix(data[,p+1]) #get the response, Y
tp=1:Nt #our observed timepoints are evenly spaced

miss=is.na(Y) #my code is capable of handling missing data
#but you'd have to tweak one line of code first
#(see run_update (fast).R file)

#read in "BUGS-style" adjacency info
source('Data/cali_adj.txt',local=TRUE)
#this next file uses the adjaceny info to make a "W" matrix
source('Functions/get.Wgen.r',local=TRUE)
#note: if you make W yourself, you need to provide a vector of 
#the number of neighbors, "num"

#this next file gives the permutation vector to go from
#\bZ=Z_1,\ldots,Z_{N_t}
#to
#\tilde{\bZ}=~Z_1,\ldots,~Z_{N_s} (or Z_M, depending on the notation)
source('Functions/Ztran.r',local=TRUE)

#plotme=FALSE #

library('mvtnorm')
library('geoR')
tj_ti=as.matrix(dist(tp,diag=TRUE,upper=TRUE)) #gets temporal distances
phimat=exp(tj_ti) #makes exponential of temporal distances

#priors and such 
as=2; bs=1 #sig2 ~ IG(as,bs)
at=2; bt=1 #tau2 ~ IG(at,bt)
ap=3/(Nt-1); bp=10 #phit ~ Unif(ap,bp)
aa=9; ab=1 #alpha ~ Beta(aa,ab)
nu=3/2 #we flirted with the idea of putting a prior on this
#but it was pointless

###########################
#Run the MCMC
###########################
set.seed(1111)
source('run_update.R',local=TRUE) #read in the code

#this code sets initial values for the chain (see 'run_update (fast).R')
get.inits(start='testing',qp=.025,qa=.025,
          phi=1,tau2=18,alpha=.9)

#using those initial values, run 10 iterations,
#save output in "Output/test1.rdata"
mod=run.up(start='testing',10,wfile='test1')
phi=mod$phi #get samples for phi
alpha=mod$alpha #etc...
tau=mod$tau2
sig=mod$sig2
Z=mod$Z
beta=mod$b
par(mfrow=c(4,5))
plot(phi,x=1:length(phi),type='l')
plot(tau,x=1:length(tau),type='l')
plot(alpha,x=1:length(alpha),type='l')
for(i in 1:p){
matplot(beta[i,],type='l')
}

#using the last value of the previous run, do 10 more iterations,
#save new output in "Output/test2.rdata"
mod=run.up(start='testing',10,wfile='test2')
phi=mod$phi
alpha=mod$alpha
tau=mod$tau2
sig=mod$sig2
Z=mod$Z
beta=mod$b
par(mfrow=c(4,5))
plot(phi,x=1:length(phi),type='l')
plot(tau,x=1:length(tau),type='l')
plot(alpha,x=1:length(alpha),type='l')
for(i in 1:p){
matplot(beta[i,],type='l')
}

#here's me showing that you can re-load in your previous chunks of samples
#
#usually, I have a loop that runs batches of 1000 iterations,
#and then I'll combine my results at the end
load(file='Output/test1.rdata')
phi=mod$phi
alpha=mod$alpha
tau=mod$tau2
sig=mod$sig2
Z=mod$Z
beta=mod$b
par(mfrow=c(4,5))
plot(phi,x=1:length(phi),type='l')
plot(tau,x=1:length(tau),type='l')
plot(alpha,x=1:length(alpha),type='l')
for(i in 1:p){
matplot(beta[i,],type='l')
}

