#################
#This is the MCMC code for the QBC paper
#(if you have missing data, search "if you have missing data..."
#Code for generating initial values is at the bottom of this file

run.up=function(start='start',T_inc=1000,wfile='current',path='',skip.a=FALSE){
#'start' is where the chain currently is (e.g., the initial values)
#'T_inc' is how many more samples you want to get
#'wfile' is where you want to save your new samples
#'path' gives the path to where you want to save them (e.g., "C:\\stuff")
#'skip.a' is a TRUE/FALSE variable for whether or not you want to *not* estimate alpha
#         (i.e., if you /don't/ want to estimate alpha, skip.a=TRUE)

#(before I started estimating alpha, I didn't have a prior on it (and thus never defined "aa" or "ab");
# this fixes that problem)
if(sum(ls()=='aa')==0){
  aa=9; ab=1
}

#start=paste('start_d',suff,k,sep=''); T_inc=10; path=paste(path,'mtau/Simulation Study #2/',sep='')

#load in current state of the chain
load(file=paste(path,'Output/',start,'.rdata',sep=''),envir=sys.frame(which=sys.nframe()))
T=mod$T+T_inc
oldT=mod$T #this should always be 1

#create arrays/vectors for our parameters, plug in current values into the first slot
b=array(dim=c(p,T))
b[,1:mod$T]=mod$b
Z=array(dim=c(M*Nt,T))
Z[,1:mod$T]=mod$Z
sig2=array(dim=c(M,T))
sig2[,1:mod$T]=mod$sig2
tau2=phi=alpha=array(dim=c(T))
tau2[1:mod$T]=mod$tau2
phi[1:mod$T]=mod$phi
alpha[1:mod$T]=mod$alpha
qt=mod$qt; qp=mod$qp; qa=mod$qa
total=mod$total

dtheta=rep(NA,T_inc)

##############
#since I'm using metropolis for alpha and phi, here's where I'm tuning the proposal densities
##############
a.accept=mod$a.accept
a.accept=ifelse(a.accept<1/6,1/6,ifelse(a.accept>0.75,0.75,a.accept))
qa=ifelse(a.accept>0.5, qa*a.accept/0.5, ifelse(a.accept<0.25, qa*a.accept/0.25, qa))
a.accept=0 #rep(0,2)

p.accept=mod$p.accept
p.accept=ifelse(p.accept<1/6,1/6,ifelse(p.accept>0.75,0.75,p.accept))
qp=ifelse(p.accept>0.5, qp*p.accept/0.5, ifelse(p.accept<0.25, qp*p.accept/0.25, qp))
p.accept=0 #rep(0,2)

##############
#plug in the current "missing" values
Ym=array(dim=c(sum(miss),T))
Ym[,1:mod$T]=mod$Ym
rm(mod)

#make the CAR precision matrix and get its determinant
DaW=D-alpha[oldT]*W
detDaW=det(DaW) #you'll want to save this for ~efficiency

####################
####################
Rphi=matern(tj_ti,1/phi[oldT],nu) #get the matern covariance matrix...
Rp=chol(Rphi) #get its cholesky...
detRp=diag(Rp) #steal the diagonal for the determinant...
Rp=chol2inv(Rp) #and get the inverse of Rphi

#permute Z,Y
Zt=(Z[,oldT])[Z2Zt] ############ no tau
Yt=Y[Z2Zt,]
Ztmat=matrix(Zt,nrow=Nt)
Ytmat=matrix(Yt,nrow=Nt)
RZ=Rp%*%Ztmat #<-permuting Z has its advantages here

for(it in (oldT+1):T){
#for(it in 2:T){
  #it=it+1

  #this reconstructs Ytmat in the case of missing data
  #(and admittedly this should be later in the code)
  if(sum(miss)>0){
    Y[miss]=Ym[,it-1]
    Yt=Y[Z2Zt,]
    Ytmat=matrix(Yt,nrow=Nt)
  }

  ###############get new betas
  Sigb=get.Sigb(sig2[,it-1])
  xyz=matrix(0,nrow=p,ncol=1)
  for(i in 1:M){
    xyz=xyz+1/sig2[i,it-1]*
        crossprod(x[((0:(Nt-1))*M+i),],(Y-Z[,it-1])[((0:(Nt-1))*M+i),1])
  }
  ub=Sigb%*%xyz
  b[,it]=rmvnorm(1,ub,Sigb)
  #b[,it]=b[,it-1]

  #############get new Z's
  for(i in 1:M){
    #THIS ASSUMES TAU2=TAU^2
    SigZ=chol(num[i]/tau2[it-1]*Rp+
              diag(rep(1/sig2[i,it-1],Nt)))
    SigZ=chol2inv(SigZ)
    muZ=alpha[it-1]/tau2[it-1]*apply(RZ[,W[i,]==1],1,sum)+
        (Ytmat[,i]-as.matrix(Xtmat[,i,],ncol=p)%*%b[,it])/sig2[i,it-1]
    muZ=SigZ%*%muZ
    Ztmat[,i]=rmvnorm(1,muZ,SigZ)
    #Ztmat[,i]=muZ
  }
  Ztmat=Ztmat-mean(Ztmat)
  Z[,it]=c(Ztmat)[Zt2Z]
  #Z[,it]=Z[,it-1]

  RZ=Rp%*%Ztmat #no taus in Ztmat
  ZRZ=t(Ztmat)%*%RZ
  ZRZW=ZRZ*W

  ##############get new sigma^2's
  #sig2[,it]=get.sig2(b[,it],Z[,it],Y)
  yxbz=Y-x%*%b[,it]-Z[,it]
 
  for(i in 1:M){
    bsp=1/bs+1/2*sum(yxbz[((0:(Nt-1))*M+i)]^2)
    sig2[i,it]=1/rgamma(1,as+Nt/2,bsp) #this is correct
  }
  #sig2[,it]=sig2[,it-1]

  ##############get new tau^2

  btp=sum(num*diag(ZRZ))-alpha[it-1]*sum(ZRZW) #it takes a few lines of algebra to get this
  btp=bt+1/2*btp
  tau2[it]=1/rgamma(1,at+Nt*M/2,btp)
  #tau2[it]=tau2[it-1]

  Zt=(Z[,it])[Z2Zt] ############ no tau
  Ztmat=matrix(Zt,nrow=Nt)

  ###############get new phi

  #get phi_star, R(phi_star), etc.
  psi=phi2psi(phi[it-1])
  psis=rnorm(1,psi,qp)
  phis=psi2phi(psis)

  Rphis=(1+phis*tj_ti)*phimat^(-phis)
  Rpsp=chol(Rphis)
  detRpsp=diag(Rpsp)
  Rpsp=chol2inv(Rpsp)
  r1=M*sum(log(detRp/detRpsp)) #no /2 because of cholesky

  Rpdiff=Rpsp-Rp
  RdiffZ=Rpdiff%*%Ztmat #Ztmat doesn't have taus in it
  ZRdiffZ=t(Ztmat)%*%RdiffZ

  r2=sum(num*diag(ZRdiffZ))-alpha[it-1]*sum(ZRdiffZ*W)

  r2=-1/(2*tau2[it])*r2
  r3=( (1+exp(psis))/(1+exp(psi)) )^2 * exp(psi)/exp(psis)

  r=exp(r1+r2)*r3 #DO WE REJECT OR ACCEPT?!?
  if(is.na(r)){ #this is mainly a debugging tool... hopefully you'll never get this error
    cat(paste('FYI: r was NaN when phi=',round(phi[it-1],4),
              'and phis=',phis,'\n',sep=''))
    r=0
  }
  accept=ifelse(r>=runif(1),1,0)
  if(accept){ #if we accept, replace these quantities with their newly accepted values
    phi[it]=phis
    Rphi=Rphis
    Rp=Rpsp
    detRp=detRpsp
  }else{ #otherwise, don't :)
    phi[it]=phi[it-1]
  }
  #phi[it]=phi[it-1]
  p.accept=p.accept+accept #keep track of how often we accept so we can tune metropolis
 
  ####################get alpha

  RZ=Rp%*%Ztmat #Ztmat doesn't have tau
  ZRZ=t(Ztmat)%*%RZ

  alpha[it]=alpha[it-1]
if(skip.a==FALSE){ #if you want to estimate alpha... (basically the same process as with phi)
  lam=logit(alpha[it-1])
  lams=rnorm(1,lam,qa) #you may want to save this sd as a parameter
  alphas=unlogit(lams)
  DaWs=D-alphas*W
  detDaWs=det(DaWs)
  r1=Nt/2*log(detDaWs/detDaW) #need the exponent
  
  r2=sum(W * ZRZ)

  r2=-(alpha[it-1]-alphas)/2 * r2/tau2[it]

  r3=exp(lams-lam) * ( (1+exp(lam))/(1+exp(lams)) )^2 *
     (alphas/alpha[it-1])^(aa-1) * ( (1-alphas)/(1-alpha[it-1]) )^(ab-1)
  r=exp(r1+r2)*r3
  accept=ifelse(r>runif(1),1,0)
  if(accept==1){
    alpha[it]=alphas
    DaW=DaWs
    detDaW=detDaWs
  }else{
    alpha[it]=alpha[it-1]
  }
}

  
  a.accept=a.accept+accept

  #when you run the code for a long time, this next chunk provides reassurance
  #that something *is* happening (albeit kinda slowly)
  if((total+it)==floor((total+it)/100)*100){
    cat(paste('Iteration: ',total+it,'\n',sep=''))
  }
  

#########################################

  #if you have missing data...
  #you need to add error to this next line
  #(the reason I haven't done it is that I'd have to loop through each region since each region has
  # its own error variance, \sigma_i^2, and I didn't want to think about that at the time)
  Ym[,it]=as.matrix(x[miss,],ncol=p)%*%b[,it]+Z[miss,it]
  Y_mu=Y-x%*%b[,it]-Z[,it]
  dtheta[it-oldT]=Nt*sum(log(sig2[,it]))+sum(Y_mu^2/sig2[,it])+M*Nt*log(2*pi)
}

#get the necessary pieces to compute DIC
Dbar=mean(dtheta)

bbar=apply(b,1,mean)
Zbar=apply(Z,1,mean)
sigbar=apply(sig2,1,mean)
Y_mu_bar=Y-x%*%bbar-Zbar
Dhat=Nt*sum(log(sigbar))+sum(Y_mu_bar^2/sigbar)+M*Nt*log(2*pi)

pD=Dbar-Dhat
DIC=Dbar+pD

total=total+T_inc

#compute acceptance rates
a.accept=a.accept/T_inc
p.accept=p.accept/T_inc

###########################
#here's where the fun starts
###########################

#save the current state of your chain so that your future runs can use it

mod=list(b=b[,T],Z=Z[,T],sig2=sig2[,T],tau2=tau2[T],alpha=alpha[T],phi=phi[T],
         a.accept=a.accept,p.accept=p.accept,qa=qa,qp=qp,T=1,total=total,Ym=Ym[,T])
save(mod,
     file=paste(path,'Output/',start,'.rdata',sep=''),
     envir=sys.frame(which=sys.nframe()))
rm(mod)

#save the entire new set of samples so that you can do whatever you want with it (and then send it back)

mod=list(b=b[,-(1:oldT)],Z=Z[,-(1:oldT)],sig2=sig2[,-(1:oldT)],tau2=tau2[-(1:oldT)],alpha=alpha[-(1:oldT)],
         phi=phi[-(1:oldT)],a.accept=a.accept,p.accept=p.accept,
         qa=qa,qp=qp,T=T_inc,total=total,Ym=Ym[,-(1:oldT)],dic=DIC,pD=pD,Dbar=Dbar,Dhat=Dhat)
save(mod,
     file=paste(path,'Output/',wfile,'.rdata',sep=''),
     envir=sys.frame(which=sys.nframe()))

return(mod)
}

##############################
#Generating initial values
##############################

get.inits=function(start='mtau',b=solve(t(x)%*%x,t(x))[,!miss]%*%Y[!miss],
                   Z=rep(0,M*Nt),sig2=rep(1.05,M),alpha=.9,
                   tau2=1.05,phi=1,T=1,a.accept=.4,p.accept=.4,
                   qp=.01,qa=.01,
                   Ym=as.matrix(x[miss,],ncol=p)%*%b+Z[miss],path=''){
#'start' is as above and 'T' should always be 1
#'b' beta vector
#'Z' random effects
#sig2 error variance parameters (NOTE: this is tau_i^2 in the paper!)
#... (skipping the obvious ones)
#a.accept is the current acceptance rate for alpha (don't bother changing this)
#p.accept "                                " phi
#qp is the standard deviation for the proposal density for phi; change as needed

#make and save the current state of the chain (i.e., the initial state)
mod=list(b=b,Z=Z,sig2=sig2,tau2=tau2,alpha=alpha,phi=phi,
         a.accept=a.accept,p.accept=p.accept,qa=qa,qp=qp,T=T,total=T,Ym=Ym[,T])
save(mod,
     file=paste(path,'Output/',start,'.rdata',sep=''))
cat('inits saved\n')
rm(mod)
}
